package cz.muni.fi.pa165.core.device;

import cz.muni.fi.pa165.core.helpers.exceptions.EntityDeletionException;
import cz.muni.fi.pa165.model.dto.device.DeviceCreateDto;
import cz.muni.fi.pa165.model.dto.device.DeviceDto;
import cz.muni.fi.pa165.model.dto.device.DeviceUpdateDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import org.springframework.data.domain.Pageable;

import javax.swing.*;
import java.util.List;

@RestController
@RequestMapping("/api/device")
@Tag(name = "Device", description = "Device API")
public class DeviceController {

    private DeviceFacade deviceFacade;

    @Autowired
    public DeviceController(DeviceFacade deviceFacade) {
      this.deviceFacade = deviceFacade;
    }


    @Operation(
            summary = "Get all devices pageable",
            description = "Returns a pageable list of all devices.")
    @GetMapping("/pageable")
    public List<DeviceDto> findAllPageable(Pageable pageable) {
      return deviceFacade.findAllPageable(pageable);
    }

    @Operation(
            summary = "Get all devices",
            description = "Returns a list of all devices.")
    @CrossOrigin(origins = "*")
    @GetMapping
    public List<DeviceDto> findAll() {
      return deviceFacade.findAll();
    }

    @Operation(
            summary = "Get device by id",
            description = "Returns a device with the specified id.")
    @GetMapping("/{id}")
    public DeviceDto findById(@PathVariable @Parameter(description = "The id of the device.") String id) {
      return deviceFacade.findById(id);
    }

    @Operation(
            summary = "Create device",
            description = "Creates a new device.")
    @PostMapping
    public DeviceDto create(@RequestBody DeviceCreateDto deviceCreateDto) {
      return deviceFacade.create(deviceCreateDto);
    }

    @Operation(
            summary = "Update device",
            description = "Updates the device with the specified id.")
    @PutMapping("/{id}")
    public DeviceDto updateById(@PathVariable @Parameter(description = "The id of the device.") String id,
                            @RequestBody @Valid DeviceUpdateDto deviceUpdateDto) {
      return deviceFacade.updateById(deviceUpdateDto, id);
    }

    @Operation(
            summary = "Delete device",
            description = "Deletes the device with the specified id.")
    @DeleteMapping("/{id}")
    public ResponseEntity<DeviceDto> deleteById(
            @PathVariable @Parameter(description = "The id of the device.") String id) {
        return new ResponseEntity<>(deviceFacade.deleteById(id), HttpStatus.OK);
    }
}
