package cz.muni.fi.pa165.core.device;

import cz.muni.fi.pa165.core.common.DomainFacade;
import cz.muni.fi.pa165.core.company.CompanyService;
import cz.muni.fi.pa165.core.helpers.exceptions.EntityDeletionException;
import cz.muni.fi.pa165.core.smartmeter.SmartMeter;
import cz.muni.fi.pa165.model.dto.device.DeviceCreateDto;
import cz.muni.fi.pa165.model.dto.device.DeviceDto;
import cz.muni.fi.pa165.model.dto.device.DeviceUpdateDto;
import cz.muni.fi.pa165.model.dto.smartDevice.SmartMeterCreateDto;
import cz.muni.fi.pa165.model.dto.smartDevice.SmartMeterDto;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class DeviceFacade extends DomainFacade<Device, DeviceDto, DeviceCreateDto, DeviceUpdateDto> {

	private final DeviceService deviceService; // For the "DeviceService" specific methods

	private final CompanyService companyService;

	@Autowired
	public DeviceFacade(DeviceService deviceService, DeviceMapper deviceMapper, CompanyService companyService) {
		super(deviceService, deviceMapper);
		this.deviceService = deviceService;
		this.companyService = companyService;
	}

	@Override
	public DeviceDto create(DeviceCreateDto deviceCreateDto){
		Device device = mapper.fromCreateDto(deviceCreateDto);
		device.setCompany(companyService.findById(deviceCreateDto.getCompanyId()));
		return mapper.toDto(deviceService.create(device));
	}

	@SneakyThrows
	@Override
	public DeviceDto deleteById(String id) {
		Device device = deviceService.findById(id);
		if (!device.getSmartMeterList().isEmpty()){
			throw new EntityDeletionException("Entity has some data");
		}
		return new DeviceDto();
	}
}
