package cz.muni.fi.pa165.core.house;

import cz.muni.fi.pa165.model.dto.house.HouseCreateDto;
import cz.muni.fi.pa165.model.dto.house.HouseDto;
import cz.muni.fi.pa165.model.dto.house.HouseUpdateDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/api/house")
@Tag(name = "House", description = "House API")

public class HouseController {

  private final HouseFacade houseFacade;

  @Autowired
  public HouseController(HouseFacade houseFacade) {
	  this.houseFacade = houseFacade;
  }

  @Operation(
          summary = "Get all houses pageable",
          description = "Retrieve a pageable list of all houses."
  )
  @GetMapping("/pageable")
  public List<HouseDto> findAllPageable(Pageable pageable) {
	  return houseFacade.findAllPageable(pageable);
  }

  @Operation(
          summary = "Get all houses",
          description = "Retrieve a list of all houses."
  )
  @CrossOrigin(origins = "*")
  @GetMapping
  public List<HouseDto> findAll() {
	  return houseFacade.findAll();
  }

  @Operation(
          summary = "Get a house by ID",
          description = "Retrieve a house with the specified ID."
  )
  @GetMapping("/{id}")
  public HouseDto findById(@PathVariable String id) {
	  return houseFacade.findById(id);
  }

  @Operation(
          summary = "Create a new house",
          description = "Create a new house with the specified details."
  )
  @PostMapping
  public HouseDto create(@RequestBody HouseCreateDto houseCreateDtoDto) {
	  return houseFacade.create(houseCreateDtoDto);
  }

  @Operation(
          summary = "Update specific house",
          description = "Updates the house with the specified id."
  )
  @PutMapping("/{id}")
  public HouseDto updateById(@RequestBody HouseUpdateDto houseUpdateDto, @PathVariable String id) {
	  return houseFacade.updateById(houseUpdateDto, id);
  }

  @Operation(
          summary = "Deletes specific house",
          description = "Deletes the house with the specified id."
  )
  @DeleteMapping("/{id}")
  public HouseDto deleteById(@PathVariable String id) {
	  return houseFacade.deleteById(id);
  }
}
