package cz.muni.fi.pa165.core.smartmeter;

import cz.muni.fi.pa165.core.common.DomainFacade;
import cz.muni.fi.pa165.core.device.DeviceService;
import cz.muni.fi.pa165.core.house.HouseService;
import cz.muni.fi.pa165.core.metrics.MetricsService;
import cz.muni.fi.pa165.model.dto.smartDevice.SmartMeterCreateDto;
import cz.muni.fi.pa165.model.dto.smartDevice.SmartMeterDto;
import cz.muni.fi.pa165.model.dto.smartDevice.SmartMeterUpdateDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

@Component
public class SmartMeterFacade extends DomainFacade<SmartMeter,
		SmartMeterDto,
		SmartMeterCreateDto,
		SmartMeterUpdateDto> {
	private final SmartMeterService smartMeterService; // For the "SmartMeterService" specific methods
	private final DeviceService deviceService;
	private final HouseService houseService;
	private final MetricsService metricsService;

	@Autowired
	public SmartMeterFacade(SmartMeterService smartMeterService,
	                        SmartMeterMapper smartMeterMapper,
	                        DeviceService deviceService, HouseService houseService, MetricsService metricsService) {
		super(smartMeterService, smartMeterMapper);
		this.smartMeterService = smartMeterService;
		this.deviceService = deviceService;
		this.houseService = houseService;
		this.metricsService = metricsService;
	}

	@Override
	public SmartMeterDto create(SmartMeterCreateDto smartMeterCreateDto) {
		SmartMeter smartMeter = mapper.fromCreateDto(smartMeterCreateDto);
		smartMeter.setDevice(deviceService.findById(smartMeterCreateDto.getDeviceId()));
		smartMeter.setHouse(houseService.findById(smartMeterCreateDto.getHouseId()));
		return mapper.toDto(smartMeterService.create(smartMeter));
	}

	@Override
	@Transactional
	public SmartMeterDto deleteById(String id) {
		// soft delete Smart Devices
		SmartMeter smartMeter = smartMeterService.findById(id);
		metricsService.deleteAllById((String[]) smartMeter.getMetricsList().stream().map(sm -> smartMeter.getId()).toArray());

		SmartMeter smartMeterDeleted = smartMeterService.deleteById(smartMeter.getId());
		return mapper.toDto(smartMeterDeleted);
	}
}
