package cz.muni.fi.pa165.core.company;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.model.dto.company.CompanyCreateDto;
import cz.muni.fi.pa165.model.dto.company.CompanyDto;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.web.servlet.MockMvc;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration tests for the {@link CompanyController} class.
 * @author xskacel
 */
@SpringBootTest
@AutoConfigureMockMvc
@TestPropertySource(locations = "classpath:application-test.properties")
public class CompanyControllerTest {

	@Autowired
	private MockMvc mockMvc;
	@Autowired
	private ObjectMapper objectMapper;

	@Autowired
	private CompanyFacade companyFacade;

	@Autowired
	private CompanyService companyService;

	private final static String URL = "/api/company";
	private final static String CONTENT_TYPE =  "application/json";

	@BeforeEach
	void setUp() {
		CompanyCreateDto companyCreateDto1 = new CompanyCreateDto();
		companyCreateDto1.setName("company uno");
		companyFacade.create(companyCreateDto1);

		CompanyCreateDto companyCreateDto2 = new CompanyCreateDto();
		companyCreateDto2.setName("company duo");
		companyFacade.create(companyCreateDto2);
	}

	@AfterEach
	void cleanUp() {
		companyService.hardDeleteAll();
	}

	/**
	 * Tests the {@link CompanyController#create(CompanyCreateDto)} method with valid input.
	 * Expects the response status code to be 201 (Created).
	 */
	@Test
	void createNonExistingCompanyTest() throws Exception {
		// Prepare
		CompanyCreateDto createDto = new CompanyCreateDto();
		createDto.setName("superDuperCompany");

		// Testing if created user is in Repository (should not be)
		assertNull(companyService.findByName(createDto.getName()));

		// Execute
		String response = mockMvc.perform(post(URL)
						.contentType(CONTENT_TYPE)
						.content(objectMapper.writeValueAsString(createDto)))
				.andExpect(status().isCreated())
				.andReturn()
				.getResponse()
				.getContentAsString();

		// Verify
		CompanyDto companyDto = objectMapper.readValue(response, CompanyDto.class);
		assertThat(companyDto.getId()).isNotNull();
		assertThat(companyDto.getName()).isEqualTo(createDto.getName());
	}
}
